@@ -7,7 +7,7 @@ use crate::connection::Authentication;
 use crate::connection_manager::{
     BrokerAddress, ConnectionManager, ConnectionRetryOptions, OperationRetryOptions, TlsOptions,
 };
-use crate::consumer::ConsumerBuilder;
+use crate::consumer::{ConsumerBuilder, ConsumerOptions, InitialPosition};
 use crate::error::Error;
 use crate::executor::Executor;
 use crate::message::proto::{self, CommandSendReceipt};
@@ -52,6 +52,14 @@ impl SerializeMessage for producer::Message {
     }
 }
 
+impl<'a> SerializeMessage for () {
+    fn serialize_message(_input: Self) -> Result<producer::Message, Error> {
+        Ok(producer::Message {
+            ..Default::default()
+        })
+    }
+}
+
 impl<'a> SerializeMessage for &'a [u8] {
     fn serialize_message(input: Self) -> Result<producer::Message, Error> {
         Ok(producer::Message {
@@ -271,6 +279,30 @@ impl<Exe: Executor> Pulsar<Exe> {
         ProducerBuilder::new(self)
     }
 
+    /// creates a reader builder
+    /// ```rust, no_run
+    /// use pulsar::reader::Reader;
+    ///
+    /// # async fn run(pulsar: pulsar::Pulsar<pulsar::TokioExecutor>) -> Result<(), pulsar::Error> {
+    /// # type TestData = String;
+    /// let mut reader: Reader<TestData, _> = pulsar
+    ///     .reader()
+    ///     .with_topic("non-persistent://public/default/test")
+    ///     .with_consumer_name("my_reader")
+    ///     .into_reader()
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn reader(&self) -> ConsumerBuilder<Exe> {
+        // this makes it exactly the same like the consumer() method though
+        ConsumerBuilder::new(self).with_options(
+            ConsumerOptions::default()
+                .durable(false)
+                .with_initial_position(InitialPosition::Latest),
+        )
+    }
+
     /// gets the address of a broker handling the topic
     ///
     /// ```rust,no_run
@@ -425,9 +457,41 @@ impl<Exe: Executor> PulsarBuilder<Exe> {
 
     /// add a custom certificate chain to authenticate the server in TLS connections
     pub fn with_certificate_chain(mut self, certificate_chain: Vec<u8>) -> Self {
-        self.tls_options = Some(TlsOptions {
-            certificate_chain: Some(certificate_chain),
-        });
+        match &mut self.tls_options {
+            Some(tls) => tls.certificate_chain = Some(certificate_chain),
+            None => {
+                self.tls_options = Some(TlsOptions {
+                    certificate_chain: Some(certificate_chain),
+                    ..Default::default()
+                })
+            }
+        }
+        self
+    }
+
+    pub fn with_allow_insecure_connection(mut self, allow: bool) -> Self {
+        match &mut self.tls_options {
+            Some(tls) => tls.allow_insecure_connection = allow,
+            None => {
+                self.tls_options = Some(TlsOptions {
+                    allow_insecure_connection: allow,
+                    ..Default::default()
+                })
+            }
+        }
+        self
+    }
+
+    pub fn with_tls_hostname_verification_enabled(mut self, enabled: bool) -> Self {
+        match &mut self.tls_options {
+            Some(tls) => tls.tls_hostname_verification_enabled = enabled,
+            None => {
+                self.tls_options = Some(TlsOptions {
+                    tls_hostname_verification_enabled: enabled,
+                    ..Default::default()
+                })
+            }
+        }
         self
     }
 
