@@ -473,6 +473,18 @@ impl<Exe: Executor> ConnectionSender<Exe> {
         .await
     }
 
+    pub async fn unsubscribe(
+        &self,
+        consumer_id: u64,
+    ) -> Result<proto::CommandSuccess, ConnectionError> {
+        let request_id = self.request_id.get();
+        let msg = messages::unsubscribe(consumer_id, request_id);
+        self.send_message(msg, RequestKey::RequestId(request_id), |resp| {
+            resp.command.success
+        })
+        .await
+    }
+
     async fn send_message<R: Debug, F>(
         &self,
         msg: Message,
@@ -1022,6 +1034,7 @@ pub(crate) mod messages {
                     encryption_algo: message.encryption_algo,
                     encryption_param: message.encryption_param,
                     schema_version: message.schema_version,
+                    deliver_at_time: message.deliver_at_time,
                     ..Default::default()
                 },
                 data: message.payload,
@@ -1208,4 +1221,18 @@ pub(crate) mod messages {
             payload: None,
         }
     }
+
+    pub fn unsubscribe(consumer_id: u64, request_id: u64) -> Message {
+        Message {
+            command: proto::BaseCommand {
+                r#type: CommandType::Unsubscribe as i32,
+                unsubscribe: Some(proto::CommandUnsubscribe {
+                    consumer_id,
+                    request_id,
+                }),
+                ..Default::default()
+            },
+            payload: None,
+        }
+    }
 }
