@@ -271,6 +271,13 @@ impl<T: DeserializeMessage, Exe: Executor> Consumer<T, Exe> {
         Ok(())
     }
 
+    pub async fn unsubscribe(&mut self) -> Result<(), Error> {
+        match &mut self.inner {
+            InnerConsumer::Single(c) => c.unsubscribe().await,
+            InnerConsumer::Multi(c) => c.unsubscribe().await,
+        }
+    }
+
     /// returns the list of topics this consumer is subscribed on
     pub fn topics(&self) -> Vec<String> {
         match &self.inner {
@@ -668,6 +675,16 @@ impl<T: DeserializeMessage, Exe: Executor> TopicConsumer<T, Exe> {
         Ok(())
     }
 
+    pub async fn unsubscribe(&mut self) -> Result<(), Error> {
+        let consumer_id = self.consumer_id;
+        self.connection()
+            .await?
+            .sender()
+            .unsubscribe(consumer_id)
+            .await?;
+        Ok(())
+    }
+
     pub fn last_message_received(&self) -> Option<DateTime<Utc>> {
         self.last_message_received
     }
@@ -1720,6 +1737,14 @@ impl<T: DeserializeMessage, Exe: Executor> MultiTopicConsumer<T, Exe> {
         Ok(())
     }
 
+    async fn unsubscribe(&mut self) -> Result<(), Error> {
+        for consumer in self.consumers.values_mut() {
+            consumer.unsubscribe().await?;
+        }
+
+        Ok(())
+    }
+
     fn add_consumers<I: IntoIterator<Item = TopicConsumer<T, Exe>>>(&mut self, consumers: I) {
         for consumer in consumers {
             let topic = consumer.topic().to_owned();
